@@ -28,6 +28,7 @@ from trac.core import *
 from trac.mimeview import Context
 from trac.perm import IPermissionRequestor
 from trac.resource import *
+from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.util.compat import set, sorted
 from trac.util.datefmt import parse_date, utc, to_timestamp, to_datetime, \
                               get_date_format_hint, get_datetime_format_hint, \
@@ -41,7 +42,7 @@ from trac.web import IRequestHandler
 from trac.web.chrome import add_link, add_stylesheet, add_warning, \
                             INavigationContributor
 from trac.wiki.api import IWikiSyntaxProvider
-from trac.wiki.formatter import format_to_html
+from trac.wiki.formatter import format_to
 
 class ITicketGroupStatsProvider(Interface):
     def get_ticket_group_stats(ticket_ids):
@@ -242,7 +243,7 @@ class DefaultTicketGroupStatsProvider(Component):
             for arg in [kv for kv in group.get('query_args', '').split(',')
                         if '=' in kv]:
                 k, v = [a.strip() for a in arg.split('=', 1)]
-                query_args[k] = v
+                query_args.setdefault(k, []).append(v)
             stat.add_interval(group.get('label', group['name']), 
                               group_cnt, query_args,
                               group.get('css_class', group['name']),
@@ -315,7 +316,7 @@ class RoadmapModule(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        return re.match(r'/roadmap/?', req.path_info) is not None
+        return req.path_info == '/roadmap'
 
     def process_request(self, req):
         milestone_realm = Resource('milestone')
@@ -471,7 +472,8 @@ class RoadmapModule(Component):
 class MilestoneModule(Component):
 
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
-               ITimelineEventProvider, IWikiSyntaxProvider, IResourceManager)
+               ITimelineEventProvider, IWikiSyntaxProvider, IResourceManager,
+               ISearchSource)
  
     stats_provider = ExtensionOption('milestone', 'stats_provider',
                                      ITicketGroupStatsProvider,
@@ -530,14 +532,13 @@ class MilestoneModule(Component):
         elif field == 'title':
             return tag('Milestone ', tag.em(milestone.id), ' completed')
         elif field == 'description':
-            return format_to_html(self.env, context(resource=milestone),
-                                  shorten_line(description))
+            return format_to(self.env, None, context(resource=milestone),
+                             description)
 
     # IRequestHandler methods
 
     def match_request(self, req):
-        import re, urllib
-        match = re.match(r'/milestone(?:/(.+))?', req.path_info)
+        match = re.match(r'/milestone(?:/(.+))?$', req.path_info)
         if match:
             if match.group(1):
                 req.args['id'] = match.group(1)
@@ -580,7 +581,7 @@ class MilestoneModule(Component):
 
         retarget_to = None
         if req.args.has_key('retarget'):
-            retarget_to = req.args.get('target')
+            retarget_to = req.args.get('target') or None
         milestone.delete(retarget_to, req.authname)
         db.commit()
         req.redirect(req.href.roadmap())
@@ -642,7 +643,7 @@ class MilestoneModule(Component):
         if milestone.exists:
             milestone.update()
             # eventually retarget opened tickets associated with the milestone
-            if 'retarget' in req.args:
+            if 'retarget' in req.args and completed:
                 cursor = db.cursor()
                 cursor.execute("UPDATE ticket SET milestone=%s WHERE "
                                "milestone=%s and status != 'closed'",
@@ -782,13 +783,15 @@ class MilestoneModule(Component):
         # should simply be false if the milestone doesn't exist in the db
         # (related to #4130)
         href = context.href.milestone(name)
-        if milestone and milestone.exists and \
-           'MILESTONE_VIEW' in context.perm(milestone.resource):
-            closed = milestone.is_completed and 'closed ' or ''
-            return tag.a(label, class_='%smilestone' % closed, href=href+extra)
-        else: 
-            return tag.a(label, class_='missing milestone', href=href+extra,
-                         rel="nofollow")
+        if milestone and milestone.exists:
+            if 'MILESTONE_VIEW' in context.perm(milestone.resource):
+                closed = milestone.is_completed and 'closed ' or ''
+                return tag.a(label, class_='%smilestone' % closed,
+                             href=href + extra)
+        elif 'MILESTONE_CREATE' in context.perm('milestone', name):
+            return tag.a(label, class_='missing milestone', href=href + extra,
+                         rel='nofollow')
+        return tag.a(label, class_='missing milestone')
         
     # IResourceManager methods
 
@@ -804,3 +807,34 @@ class MilestoneModule(Component):
             return self._render_link(context, resource.id, desc)
         else:
             return desc
+
+    # ISearchSource methods
+
+    def get_search_filters(self, req):
+        if 'MILESTONE_VIEW' in req.perm:
+            yield ('milestone', _('Milestones'))
+
+    def get_search_results(self, req, terms, filters):
+        if not 'milestone' in filters:
+            return
+        db = self.env.get_db_cnx()
+        sql_query, args = search_to_sql(db, ['name', 'description'], terms)
+        cursor = db.cursor()
+        cursor.execute("SELECT name,due,completed,description "
+                       "FROM milestone "
+                       "WHERE " + sql_query, args)
+
+        milestone_realm = Resource('milestone')
+        for name, due, completed, description in cursor:
+            milestone = milestone_realm(id=name)
+            if 'MILESTONE_VIEW' in req.perm(milestone):
+                yield (get_resource_url(self.env, milestone, req.href),
+                       get_resource_name(self.env, milestone),
+                       datetime.fromtimestamp(
+                           completed or due or time(), utc),
+                       '', shorten_result(description, terms))
+        
+        # Attachments
+        for result in AttachmentModule(self.env).get_search_results(
+            req, milestone_realm, terms):
+            yield result
