@@ -24,6 +24,7 @@ from genshi.core import Markup
 from genshi.builder import tag
 
 from trac.attachment import AttachmentModule
+from trac.config import IntOption
 from trac.core import *
 from trac.mimeview.api import Mimeview, IContentConverter, Context
 from trac.perm import IPermissionRequestor
@@ -40,7 +41,7 @@ from trac.web.chrome import add_link, add_script, add_stylesheet, \
                             INavigationContributor, ITemplateProvider
 from trac.web import IRequestHandler
 from trac.wiki.api import IWikiPageManipulator, WikiSystem
-from trac.wiki.formatter import format_to_oneliner
+from trac.wiki.formatter import format_to
 from trac.wiki.model import WikiPage
  
 class InvalidWikiPage(TracError):
@@ -58,6 +59,9 @@ class WikiModule(Component):
 
     page_manipulators = ExtensionPoint(IWikiPageManipulator)
 
+    max_size = IntOption('wiki', 'max_size', 262144,
+        """Maximum allowed wiki page size in bytes. (''since 0.11.2'')""")
+
     PAGE_TEMPLATES_PREFIX = 'PageTemplates/'
     DEFAULT_PAGE_TEMPLATE = 'DefaultPage'
 
@@ -96,7 +100,7 @@ class WikiModule(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        match = re.match(r'^/wiki(?:/(.*)|$)', req.path_info)
+        match = re.match(r'/wiki(?:/(.+))?$', req.path_info)
         if match:
             if match.group(1):
                 req.args['page'] = match.group(1)
@@ -173,6 +177,14 @@ class WikiModule(Component):
 
     def _validate(self, req, page):
         valid = True
+        
+        # Validate page size
+        if len(req.args.get('text', '')) > self.max_size:
+            add_warning(req, _('The wiki page is too long (must be less '
+                               'than %(num)s characters)',
+                               num=self.max_size))
+            valid = False
+
         # Give the manipulators a pass at post-processing the page
         for manipulator in self.page_manipulators:
             for field, message in manipulator.validate_wiki_page(req, page):
@@ -264,9 +276,8 @@ class WikiModule(Component):
             page.save(get_reporter_id(req, 'author'),
                             req.args.get('comment'),
                             req.remote_addr)
-            not_modified = False
             req.redirect(get_resource_url(self.env, page.resource, req.href,
-                                          version=page.version))
+                                          version=None))
         except TracError:
             add_warning(req, _("Page not modified, showing latest version."))
             return self._render_view(req, page)
@@ -457,15 +468,16 @@ class WikiModule(Component):
         version = page.resource.version
 
         # Add registered converters
-        for conversion in Mimeview(self.env).get_supported_conversions(
-                                             'text/x-trac-wiki'):
-            conversion_href = req.href.wiki(page.name, version=version,
-                                            format=conversion[0])
-            # or...
-            conversion_href = get_resource_url(self.env, page.resource,
-                                               req.href, format=conversion[0])
-            add_link(req, 'alternate', conversion_href, conversion[1],
-                     conversion[3])
+        if page.exists:
+            for conversion in Mimeview(self.env).get_supported_conversions(
+                                                 'text/x-trac-wiki'):
+                conversion_href = req.href.wiki(page.name, version=version,
+                                                format=conversion[0])
+                # or...
+                conversion_href = get_resource_url(self.env, page.resource,
+                                                req.href, format=conversion[0])
+                add_link(req, 'alternate', conversion_href, conversion[1],
+                         conversion[3])
 
         data = self._page_data(req, page)
         if page.name == 'WikiStart':
@@ -506,8 +518,14 @@ class WikiModule(Component):
                      req.href.wiki(page.name, version=prev_version),
                      _('Version %(num)s', num=prev_version))
 
-        add_link(req, 'up', req.href.wiki(page.name, version=None),
-                 _('View Latest Version'))
+        parent = None
+        if version:
+            add_link(req, 'up', req.href.wiki(page.name, version=None),
+                     _('View latest version'))
+        elif '/' in page.name:
+            parent = page.name[:page.name.rindex('/')]
+            add_link(req, 'up', req.href.wiki(parent, version=None),
+                     _("View parent page"))
         
         if next_version:
             add_link(req, 'next',
@@ -521,6 +539,8 @@ class WikiModule(Component):
                         req.href.wiki(page.name, action='diff',
                                       version=page.version))
         else:
+            if parent:
+                add_ctxtnav(req, _('Up'), req.href.wiki(parent))
             self._wiki_ctxtnav(req, page)
 
         context = Context.from_request(req, page.resource)
@@ -579,10 +599,8 @@ class WikiModule(Component):
             return tag(tag.em(get_resource_name(self.env, wiki_page)),
                        wiki_page.version > 1 and ' edited' or ' created')
         elif field == 'description':
-            if self.config['timeline'].getbool('abbreviated_messages'):
-                comment = shorten_line(comment)
-            markup = format_to_oneliner(self.env, context(resource=wiki_page),
-                                        comment)
+            markup = format_to(self.env, None, context(resource=wiki_page),
+                               comment)
             if wiki_page.version > 1:
                 diff_href = context.href.wiki(
                     wiki_page.id, version=wiki_page.version, action='diff')
@@ -617,3 +635,8 @@ class WikiModule(Component):
                        '%s: %s' % (name, shorten_line(text)),
                        datetime.fromtimestamp(ts, utc), author,
                        shorten_result(text, terms))
+        
+        # Attachments
+        for result in AttachmentModule(self.env).get_search_results(
+            req, wiki_realm, terms):
+            yield result
